/*
 * Copyright (C) Centeris Corporation 2004-2007
 * Copyright (C) Likewise Software 2007.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _CENTERROR_H
#define _CENTERROR_H

/*
 * Likewise software error code definitions.
 *
 * Current list of components with specific error codes:
 *     Group Policy component of Likewise Identity
 *     Some other component of Likewise Identity
 *
 * Error code field structure format:
 *     32 bit length total length
 *         0x00000000 - 0x0000FFFF = Likewise System error range
 *         0x00010000 - 0x0001FFFF = Likewise ErrNo error range
 *         0x00050000 - 0x0005FFFF = Likewise Group Policy error range
 *         0x00070000 - 0x0007FFFF = Likewise Registry CSE error range
 *         0x00080000 - 0x0008FFFF = Likewise Domainjoin error range
 *         0x00060000 - 0x000FFFFF = Reserved for future Likewise software components
 */


typedef uint32_t CENTERROR;


#define CENTERROR_IS_OK(x) (x == 0)
#define CENTERROR_EQUAL(x,y) (x == y)

#define CENTERROR_COMPONENT(x) ((x & 0x0FFF0000) >> 16)

#define CENTERROR_COMPONENT_SYSTEM     0x000 /* Likewise software System */
#define CENTERROR_COMPONENT_ERRNO      0x001 /* Likewise software ErrNo */
#define CENTERROR_COMPONENT_GP         0x005 /* Likewise software Group Policy Agent (Centeris-GPAgent) */
#define CENTERROR_COMPONENT_REGISTRY   0x007 /* Likewise software Elektra */
#define CENTERROR_COMPONENT_DOMAINJOIN 0x008 /* Likewise software Domainjoin */

/*
 * #define CENTERROR_COMPONENT_Other  0x006 // Example: Other Likewise software Component
 */

/*
 * Likewise software errno error macros
 */
#define CENTERROR_ERRNO_MASK (CENTERROR_COMPONENT_ERRNO << 16)
#define CENTERROR_ERRNO(code) (CENTERROR_ERRNO_MASK | ((code) & 0xFFFF))
#define CENTERROR_IS_ERRNO(x) ((x & 0x0FFF0000) == CENTERROR_ERRNO_MASK)
#define CENTERROR_ERRNO_CODE(x) (x & 0x0000FFFF)
/*
 * Likewise software system error macros
 */
#define CENTERROR_SYSTEM_MASK (CENTERROR_COMPONENT_SYSTEM << 16)
#define CENTERROR_SYSTEM(code) (CENTERROR_SYSTEM_MASK | code)
#define CENTERROR_IS_SYSTEM(x) ((x & 0x0FFF0000) == CENTERROR_SYSTEM_MASK)
#define CENTERROR_SYSTEM_CODE(x) (x & 0x0000FFFF)

/*
 * Group Policy error macros
 */
#define CENTERROR_GP_MASK (CENTERROR_COMPONENT_GP << 16)
#define CENTERROR_GP(code) (CENTERROR_GP_MASK | code)
#define CENTERROR_IS_GP(x) ((x & 0x0FFF0000) == CENTERROR_GP_MASK)
#define CENTERROR_GP_CODE(x) (x & 0x0000FFFF)

/*
 * DomainJoin error macros
 */
#define CENTERROR_DOMAINJOIN_MASK (CENTERROR_COMPONENT_DOMAINJOIN << 16)
#define CENTERROR_DOMAINJOIN(code) (CENTERROR_DOMAINJOIN_MASK | code)
#define CENTERROR_IS_DOMAINJOIN(x) ((x & 0x0FFF0000) == CENTERROR_DOMAINJOIN_MASK)
#define CENTERROR_DOMAINJOIN_CODE(x) (x & 0x0000FFFF)

#define CENTERROR_SUCCESS                       0x00000000                    //      0
#define CENTERROR_ACCESS_DENIED                 CENTERROR_SYSTEM(0x00000005)  //      5
#define CENTERROR_OUT_OF_MEMORY                 CENTERROR_SYSTEM(0x0000000E)  //     14
#define CENTERROR_INVALID_PASSWORD              CENTERROR_SYSTEM(0x00000056)  //     86
#define CENTERROR_INVALID_PARAMETER             CENTERROR_SYSTEM(0x00000057)  //     87
#define CENTERROR_INVALID_OPERATION             CENTERROR_SYSTEM(0x00000058)  //     88
#define CENTERROR_INVALID_MESSAGE               CENTERROR_SYSTEM(0x000003EA)  //   1002
#define CENTERROR_CONNECTION_UNAVAIL            CENTERROR_SYSTEM(0x000004B1)  //   1201
#define CENTERROR_INVALID_GROUPNAME             CENTERROR_SYSTEM(0x000004B9)  //   1209
#define CENTERROR_INVALID_COMPUTERNAME          CENTERROR_SYSTEM(0x000004BA)  //   1210
#define CENTERROR_INVALID_DOMAINNAME            CENTERROR_SYSTEM(0x000004BC)  //   1212
#define CENTERROR_INVALID_SERVICENAME           CENTERROR_SYSTEM(0x000004BD)  //   1213
#define CENTERROR_INVALID_NETNAME               CENTERROR_SYSTEM(0x000004BE)  //   1214
#define CENTERROR_INVALID_SHARENAME             CENTERROR_SYSTEM(0x000004BF)  //   1215
#define CENTERROR_INVALID_PASSWORDNAME          CENTERROR_SYSTEM(0x000004C0)  //   1216
#define CENTERROR_INVALID_VALUE                 CENTERROR_SYSTEM(0x000004C1)  //   1217
#define CENTERROR_INVALID_VALUE_TYPE            CENTERROR_SYSTEM(0x000004C2)  //   1218
#define CENTERROR_DUP_DOMAINNAME                CENTERROR_SYSTEM(0x000004C5)  //   1221
#define CENTERROR_NO_NETWORK                    CENTERROR_SYSTEM(0x000004C6)  //   1222
#define CENTERROR_CONNECTION_REFUSED            CENTERROR_SYSTEM(0x000004C9)  //   1225
#define CENTERROR_CONNECTION_INVALID            CENTERROR_SYSTEM(0x000004CD)  //   1229
#define CENTERROR_ILL_FORMED_PASSWORD           CENTERROR_SYSTEM(0x0000052C)  //   1324
#define CENTERROR_PASSWORD_RESTRICTION          CENTERROR_SYSTEM(0x0000052D)  //   1325
#define CENTERROR_LOGON_FAILURE                 CENTERROR_SYSTEM(0x0000052E)  //   1326
#define CENTERROR_PASSWORD_EXPIRED              CENTERROR_SYSTEM(0x00000532)  //   1330
#define CENTERROR_ACCOUNT_DISABLED              CENTERROR_SYSTEM(0x00000533)  //   1331
#define CENTERROR_BAD_TOKEN                     CENTERROR_SYSTEM(0x00000545)  //   1349
#define CENTERROR_BAD_GUID                      CENTERROR_SYSTEM(0x00000546)  //   1350
#define CENTERROR_NO_SUCH_DOMAIN                CENTERROR_SYSTEM(0x0000054B)  //   1355
#define CENTERROR_DOMAIN_EXISTS                 CENTERROR_SYSTEM(0x0000054C)  //   1356
#define CENTERROR_DOMAIN_JOIN_FAILED            CENTERROR_SYSTEM(0x0000054D)  //   1357
#define CENTERROR_DOMAIN_LEAVE_FAILED           CENTERROR_SYSTEM(0x0000054E)  //   1358
#define CENTERROR_LICENSE_INCORRECT             CENTERROR_SYSTEM(0x00002001)  //   8193
#define CENTERROR_LICENSE_EXPIRED               CENTERROR_SYSTEM(0x00002002)  //   8194
#define CENTERROR_LICENSE_ALREADY_IN_USE        CENTERROR_SYSTEM(0x00002003)  //   8195
#define CENTERROR_LICENSE_NOT_AVAILABLE_FOR_USE CENTERROR_SYSTEM(0x00002004)  //   8196
#define CENTERROR_LICENSE_AD_NOT_PROVISIONED    CENTERROR_SYSTEM(0x00002005)  //   8197
#define CENTERROR_LICENSE_DUPLICATE_PROVISIONED CENTERROR_SYSTEM(0x00002006)  //   8198
#define CENTERROR_LICENSE_NO_KEY_IN_DN          CENTERROR_SYSTEM(0x00002007)  //   8199
#define CENTERROR_CFG_INVALID_SECTION_NAME      CENTERROR_SYSTEM(0x00002008)  //   8200
#define CENTERROR_CFG_INVALID_NVPAIR_NAME       CENTERROR_SYSTEM(0x00002009)  //   8201
#define CENTERROR_CFG_SECTION_NOT_FOUND         CENTERROR_SYSTEM(0x0000200A)  //   8202
#define CENTERROR_CFG_VALUE_NOT_FOUND           CENTERROR_SYSTEM(0x0000200B)  //   8203
#define CENTERROR_VALUE_NOT_IN_SECTION          CENTERROR_SYSTEM(0x0000200C)  //   8204
#define CENTERROR_CFG_INVALID_SIGNATURE         CENTERROR_SYSTEM(0x0000200D)  //   8205
#define CENTERROR_CFG_NOT_ENOUGH_BUFFER         CENTERROR_SYSTEM(0x0000200E)  //   8206
#define CENTERROR_REGEX_COMPILE_FAILED          CENTERROR_SYSTEM(0x0000200F)  //   8207
#define CENTERROR_LWIAUTH_NOT_RUNNING           CENTERROR_SYSTEM(0x00002010)  //   8208
#define CENTERROR_NO_SUCH_PROCESS               CENTERROR_SYSTEM(0x00002011)  //   8209
#define CENTERROR_NO_SUCH_SYMBOL                CENTERROR_SYSTEM(0x00002012)  //   8210
#define CENTERROR_PTHREAD_ERROR                 CENTERROR_SYSTEM(0x00002013)  //   8211
#define CENTERROR_COMMAND_FAILED                CENTERROR_SYSTEM(0x00002014)  //   8212
#define CENTERROR_INVALID_UID                   CENTERROR_SYSTEM(0x00002015)  //   8213
#define CENTERROR_INVALID_GID                   CENTERROR_SYSTEM(0x00002016)  //   8214
#define CENTERROR_INVALID_FILENAME              CENTERROR_SYSTEM(0x00002017)  //   8215
//An option passed on the command line was invalid
#define CENTERROR_INVALID_OPTION_VALUE          CENTERROR_SYSTEM(0x00002018)  //   8216
#define CENTERROR_ABNORMAL_TERMINATION          CENTERROR_SYSTEM(0x00002019)  //   8217
#define CENTERROR_SED_NOT_FOUND	                CENTERROR_SYSTEM(0x00002020)  //   8218
#define CENTERROR_SHELL_VARIABLE_UNKNOWN        CENTERROR_SYSTEM(0x00002021)  //   8219
#define CENTERROR_FILE_NOT_FOUND        	CENTERROR_SYSTEM(0x00002022)  //   8220
#define CENTERROR_NOT_IMPLEMENTED        	CENTERROR_SYSTEM(0x00002023)  //   8221
#define CENTERROR_INVALID_DIRECTORY             CENTERROR_SYSTEM(0x00002024)  //   8222

#define CENTERROR_GP_FILE_OPEN_FAILED           CENTERROR_GP(0x00000020)      // 327712

#define CENTERROR_DOMAINJOIN_NON_ROOT_USER         CENTERROR_DOMAINJOIN(0x00000001) // 524289
#define CENTERROR_DOMAINJOIN_INVALID_HOSTNAME      CENTERROR_DOMAINJOIN(0x00000002) // 524290
#define CENTERROR_DOMAINJOIN_SYSCONFIG_EDIT_FAIL   CENTERROR_DOMAINJOIN(0x00000003) // 524291
#define CENTERROR_DOMAINJOIN_NO_ETH_ITF_CFG_FILE   CENTERROR_DOMAINJOIN(0x00000004) // 524292
#define CENTERROR_DOMAINJOIN_DHCPINFO_EDIT_FAIL    CENTERROR_DOMAINJOIN(0x00000005) // 524293
#define CENTERROR_DOMAINJOIN_HOSTS_EDIT_FAIL       CENTERROR_DOMAINJOIN(0x00000006) // 524294
#define CENTERROR_DOMAINJOIN_SET_MACHINESID_FAIL   CENTERROR_DOMAINJOIN(0x00000007) // 524295
#define CENTERROR_DOMAINJOIN_NETCONFIGCMD_FAIL     CENTERROR_DOMAINJOIN(0x00000008) // 524296
#define CENTERROR_DOMAINJOIN_DHCPRESTART_SET_FAIL  CENTERROR_DOMAINJOIN(0x00000009) // 524297
#define CENTERROR_DOMAINJOIN_DHCPRESTART_FAIL      CENTERROR_DOMAINJOIN(0x0000000A) // 524298
#define CENTERROR_DOMAINJOIN_KRB5_EDIT_FAIL        CENTERROR_DOMAINJOIN(0x0000000B) // 524299
#define CENTERROR_DOMAINJOIN_NSSWITCH_EDIT_FAIL    CENTERROR_DOMAINJOIN(0x0000000C) // 524300
#define CENTERROR_DOMAINJOIN_PAM_EDIT_FAIL         CENTERROR_DOMAINJOIN(0x0000000D) // 524301
#define CENTERROR_DOMAINJOIN_JOIN_FAILED           CENTERROR_DOMAINJOIN(0x0000000E) // 524302
#define CENTERROR_DOMAINJOIN_JOIN_TIMEDOUT         CENTERROR_DOMAINJOIN(0x0000000F) // 524303
#define CENTERROR_DOMAINJOIN_JOIN_NO_WKGRP         CENTERROR_DOMAINJOIN(0x00000010) // 524304
#define CENTERROR_DOMAINJOIN_SMB_VALUE_NOT_FOUND   CENTERROR_DOMAINJOIN(0x00000011) // 524305
#define CENTERROR_DOMAINJOIN_DOMAIN_NOT_FOUND      CENTERROR_DOMAINJOIN(0x00000012) // 524306
#define CENTERROR_DOMAINJOIN_WORKGROUP_NOT_FOUND   CENTERROR_DOMAINJOIN(0x00000013) // 524307
#define CENTERROR_DOMAINJOIN_DESCRIPTION_NOT_FOUND CENTERROR_DOMAINJOIN(0x00000014) // 524308
#define CENTERROR_DOMAINJOIN_HOSTNAME_CONTAINS_DOT CENTERROR_DOMAINJOIN(0x00000015) // 524309
#define CENTERROR_DOMAINJOIN_MISSING_DAEMON        CENTERROR_DOMAINJOIN(0x00000016) // 524310
#define CENTERROR_DOMAINJOIN_UNEXPECTED_ERRCODE    CENTERROR_DOMAINJOIN(0x00000017) // 524311
#define CENTERROR_DOMAINJOIN_INCORRECT_STATUS      CENTERROR_DOMAINJOIN(0x00000018) // 524312
#define CENTERROR_DOMAINJOIN_CHKCONFIG_FAILED      CENTERROR_DOMAINJOIN(0x00000019) // 524313
#define CENTERROR_DOMAINJOIN_UPDATERCD_FAILED      CENTERROR_DOMAINJOIN(0x0000001A) // 524314
#define CENTERROR_DOMAINJOIN_LICENSE_ERROR         CENTERROR_DOMAINJOIN(0x0000001B) // 524315
#define CENTERROR_DOMAINJOIN_NO_LICENSE_KEY        CENTERROR_DOMAINJOIN(0x0000001C) // 524316
#define CENTERROR_DOMAINJOIN_WRONG_PRODUCT_CODE    CENTERROR_DOMAINJOIN(0x0000001D) // 524317
#define CENTERROR_DOMAINJOIN_WRONG_LICENSE_VERSION CENTERROR_DOMAINJOIN(0x0000001E) // 524318
#define CENTERROR_DOMAINJOIN_NOT_JOINED_TO_AD      CENTERROR_DOMAINJOIN(0x0000001F) // 524319
#define CENTERROR_DOMAINJOIN_GPAGENTD_INCOMMUNICADO CENTERROR_DOMAINJOIN(0x00000020) // 524320
#define CENTERROR_DOMAINJOIN_BAD_LICENSE_KEY        CENTERROR_DOMAINJOIN(0x00000021) // 524321
#define CENTERROR_DOMAINJOIN_INVALID_USERID         CENTERROR_DOMAINJOIN(0x00000022) // 524322
#define CENTERROR_DOMAINJOIN_INVALID_DOMAIN_NAME    CENTERROR_DOMAINJOIN(0x00000023) // 524323
#define CENTERROR_DOMAINJOIN_INVALID_PASSWORD       CENTERROR_DOMAINJOIN(0x00000024) // 524324
#define CENTERROR_DOMAINJOIN_FAILED_TO_LEAVE_DOMAIN CENTERROR_DOMAINJOIN(0x00000025) // 524325
#define CENTERROR_DOMAINJOIN_UNRESOLVED_DOMAIN_NAME CENTERROR_DOMAINJOIN(0x00000026) // 524326
#define CENTERROR_DOMAINJOIN_INVALID_LOG_LEVEL      CENTERROR_DOMAINJOIN(0x00000027) // 524327
#define CENTERROR_DOMAINJOIN_FAILED_SET_HOSTNAME    CENTERROR_DOMAINJOIN(0x00000028) // 524328
#define CENTERROR_DOMAINJOIN_LISTSVCS_FAILED        CENTERROR_DOMAINJOIN(0x00000029) // 524329
#define CENTERROR_DOMAINJOIN_SVC_LOAD_FAILED        CENTERROR_DOMAINJOIN(0x0000002A) // 524330
#define CENTERROR_DOMAINJOIN_SVC_UNLOAD_FAILED      CENTERROR_DOMAINJOIN(0x0000002B) // 524331
#define CENTERROR_DOMAINJOIN_PREPSVC_FAILED         CENTERROR_DOMAINJOIN(0x0000002C) // 524332
#define CENTERROR_DOMAINJOIN_UNKNOWN_DAEMON         CENTERROR_DOMAINJOIN(0x0000002D) // 524333
#define CENTERROR_DOMAINJOIN_INVALID_OU             CENTERROR_DOMAINJOIN(0x0000002E) // 524334
#define CENTERROR_DOMAINJOIN_FAILED_KICKER_INSTALL   CENTERROR_DOMAINJOIN(0x0000002F) // 524335
#define CENTERROR_DOMAINJOIN_FAILED_KICKER_UNINSTALL CENTERROR_DOMAINJOIN(0x00000030) // 524336
#define CENTERROR_DOMAINJOIN_FAILED_SET_SEARCHPATH CENTERROR_DOMAINJOIN(0x00000031) // 524337
#define CENTERROR_DOMAINJOIN_FAILED_REG_OPENDIR    CENTERROR_DOMAINJOIN(0x00000032) // 524338
#define CENTERROR_DOMAINJOIN_FAILED_UNREG_OPENDIR  CENTERROR_DOMAINJOIN(0x00000033) // 524339
#define CENTERROR_DOMAINJOIN_PAM_MISSING_SERVICE   CENTERROR_DOMAINJOIN(0x00000034) // 524340
#define CENTERROR_DOMAINJOIN_PAM_BAD_CONF          CENTERROR_DOMAINJOIN(0x00000035) // 524341
#define CENTERROR_DOMAINJOIN_PAM_MISSING_MODULE    CENTERROR_DOMAINJOIN(0x00000036) // 524342
#define CENTERROR_DOMAINJOIN_FAILED_ADMIN_PRIVS    CENTERROR_DOMAINJOIN(0x00000037) // 524343
#define CENTERROR_DOMAINJOIN_LWINET_TIME_FAILED    CENTERROR_DOMAINJOIN(0x00000038) // 524344
#define CENTERROR_DOMAINJOIN_TIME_NOT_SET	   CENTERROR_DOMAINJOIN(0x00000039) // 524345
#define CENTERROR_DOMAINJOIN_INVALID_FORMAT	   CENTERROR_DOMAINJOIN(0x00000040) // 524346
#define CENTERROR_DOMAINJOIN_INVALID_FIREWALLCFG   CENTERROR_DOMAINJOIN(0x00000041) // 524347

const char*
CTErrorName(
    CENTERROR error
    );

const char*
CTErrorDescription(
    CENTERROR error
    );

const char*
CTErrorHelp(
    CENTERROR error
    );

CENTERROR
CTErrorFromName(
    const char* name
    );

CENTERROR
CTMapSystemError(
    int dwError
    );

#endif
